window.Gleam.initSearch([{"doc":"signal_pgo","title":"signal_pgo","content":"# signal_pgo\n\n[![Package Version](https://img.shields.io/hexpm/v/signal_pgo)](https://hex.pm/packages/signal_pgo)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/signal_pgo/)\n\n```sh\ngleam add signal signal_pgo\n```\n\nThis is a persistance layer for the signal event sourcing library, on application startup, it will run migrations to set up an events table, and then it will be able to store and load events from signal. To configure it, you need to provide a gleam_pgo config, as well as an encoder and decoder for your events.\n\nThe encoding and decoding is done to a String, because event data is stored as a TEXT field in postgres. This also allows for maximum flexibility, allowing you to control exactly how data is stored.\n\nThis flexibility lets you implement things like encryption, or avoid the performance overhead of JSON parsing.\n\n## Usage\n\nSet up a local database:\n\n```sh\ndocker run --name your-db-name -e POSTGRES_USER=postgres -e POSTGRES_PASSWORD=postgres -p 5500:5432 -d postgres\n```\n\nInitiate the Signal_PGO and Signal, configure them:\n\n```gleam\npub fn main() {\n\n    // Create your typical Gleam PGO database config in your config code\n    let pgo_config =\n        pgo.Config(\n            ..pgo.default_config(),\n            host: &quot;localhost&quot;,\n            port: 5500,\n            database: &quot;signal_dev&quot;,\n            user: &quot;postgres&quot;,\n            password: option.Some(&quot;postgres&quot;),\n            pool_size: 2,\n        )\n\n    // Start the Signal_PGO services, this will return two actors\n    // First actor is the one you use to configure Signal\n    // Second actor is a helper that saves and retrieves String blobs,\n    //      useful for persisting your event consumer state, as sort of a projection.\n    //\n    let assert Ok(#(aggregate_persistance, _projection_persistance)) =\n        |&gt; signal_pgo.start(pgo_config, your_event_encoder, your_event_decoder)\n\n    let aggregate_configuration =\n        signal.AggregateConfig(\n        initial_state: your_aggregate_initial_state,\n        command_handler: your_command_handler,\n        event_handler: your_event_handler,\n        )\n\n    // Use the with_persistance to configure Signal with this persistance layer\n    let assert Ok(es_service) =\n        signal.configure(aggregate_configuration)\n        |&gt; signal.with_persistence(aggregate_persistance)\n        |&gt; signal.with_subscriber(signal.Consumer(revenue_report))\n        |&gt; signal.start()\n\n}\n```\n\nYour application will now store events produced by the aggregate in the database, and use it to hydrate aggregates when needed.\n\n## Projection store\n\nSignal PGO will create a projection store table which can store any blobs of TEXT data. This is a rudimentary helper to help you store your projection state. The end goal is to offer proper projections functionality in Signal in the future, but for the time being you have to handle most of the logic yourself.\n","url":"index.html"},{"doc":"signal_pgo","title":"Projection","content":"pub type Projection {\n  Projection(\n    projection_id: String,\n    projection_name: String,\n    projection_version: Int,\n    data: String,\n  )\n}\n\nProjection(\n  projection_id: String,\n  projection_name: String,\n  projection_version: Int,\n  data: String,\n)\n\n\nSynonyms:\nsignal_pgo.Projection\nsignal_pgo Projection","url":"signal_pgo.html#Projection"},{"doc":"signal_pgo","title":"ProjectionMessage","content":"pub type ProjectionMessage {\n  Store(Projection)\n  Get(\n    reply_with: process.Subject(Result(Projection, String)),\n    String,\n  )\n  Delete(String)\n  Shutdown\n}\n\nStore(Projection)\n\n\nGet(\n  reply_with: process.Subject(Result(Projection, String)),\n  String,\n)\n\n\nDelete(String)\n\n\nShutdown\n\n\nSynonyms:\nsignal_pgo.ProjectionMessage\nsignal_pgo ProjectionMessage","url":"signal_pgo.html#ProjectionMessage"},{"doc":"signal_pgo","title":"start","content":"pub fn start(\n  pgo_config: Config,\n  event_encoder: fn(a) -&gt; String,\n  event_decoder: fn(String, String) -&gt; Result(a, String),\n) -&gt; Result(\n  #(Subject(StoreMessage(a)), Subject(ProjectionMessage)),\n  String,\n)\n\nSynonyms:\nsignal_pgo.start\nsignal_pgo start","url":"signal_pgo.html#start"},{"doc":"signal_pgo","title":"signal_pgo","content":"","url":"signal_pgo.html"}]);